<?php

if ( ! defined( 'ABSPATH' ) ) { 
    exit; // Exit if accessed directly
}

class WC_QD_Vat_Number_Field {

	const META_KEY = 'vat_number';

	/**
	 * Setup
	 *
	 * @since 1.0
	 */
	public function setup() {
		add_action( 'woocommerce_after_checkout_billing_form', array( $this, 'print_field' ) );
		add_action( 'woocommerce_checkout_update_order_meta', array( $this, 'save_field' ) );
		add_action( 'woocommerce_after_checkout_validation', array( $this, 'validate_field' ), 1 );
		add_action( 'woocommerce_admin_order_data_after_billing_address', array( $this, 'display_field' ), 10, 1 );
		add_filter( 'woocommerce_form_field_hidden', array( $this, 'wc_form_hidden_field' ), 10, 4 );

    add_action( 'show_user_profile', array( $this, 'add_customer_meta_fields'), 30, 1 ); 
    add_action( 'edit_user_profile', array( $this, 'add_customer_meta_fields'), 30, 1 ); 

    add_action( 'personal_options_update', array( $this, 'save_customer_meta_fields'), 30, 1 ); 
    add_action( 'edit_user_profile_update', array( $this, 'save_customer_meta_fields'), 30, 1 ); 
	}

	/**
	 * Print the VAT field
	 *
	 * @since 1.0
	 */
	public function print_field() {
	  global $woocommerce;
    $user_vat_number = get_user_meta( get_current_user_id(), '_quaderno_vat_number', true );

		woocommerce_form_field( 'vat_number', array(
			'type'   => 'text',
			'label'  => esc_html__( 'VAT number', 'woocommerce-quaderno' ),
			'class'  => array( 'update_totals_on_change' ),
      'autocomplete' => 'nope',
		), $user_vat_number );

		woocommerce_form_field( 'base_location', array(
			'type'   => 'hidden',
			'default' => $woocommerce->countries->get_base_country()
		));
	}

	/**
	 * Save the VAT number to the order
	 *
	 * @param $order_id
	 */
	public function save_field( $order_id ) {
    global $woocommerce;

    $billing_country = WC()->customer->get_billing_country();
    $shop_country = $woocommerce->countries->get_base_country();

		if ( ! empty( $_POST['vat_number'] ) && $billing_country != $shop_country ) {
			// Save the VAT number
			update_post_meta( $order_id, self::META_KEY, sanitize_text_field( $_POST['vat_number'] ) );

			// Reset the customer VAT exempt state
			WC()->customer->set_is_vat_exempt( false );
		}
	}

	/**
	 * Validate the VAT field
	 *
	 * @since 1.4
	 */
	public function validate_field() {
    global $woocommerce;

    $billing_country = WC()->customer->get_billing_country();
    $shop_country = $woocommerce->countries->get_base_country();

		if ( ! empty( $_POST['vat_number'] ) && $billing_country != $shop_country ) {
		  $valid_number = $this::is_valid( $_POST['vat_number'], $billing_country );

			if ( false === $valid_number ) {
				wc_add_notice( sprintf( esc_html__( '%s is not valid.', 'woocommerce-quaderno' ), '<strong>' . esc_html__( 'VAT number', 'woocommerce-quaderno' ) . '</strong>' ), 'error' );
			}
		}
	}

	/**
	 * Display the VAT field in the backend
	 *
	 * @param $order
	 */
	public function display_field( $order ) {
		$vat_number = get_post_meta( $order->get_id(), self::META_KEY, true );
		if ( '' != $vat_number ) {
			echo '<p><strong style="display:block;">' . esc_html__( 'VAT number', 'woocommerce-quaderno' ) . ':</strong> ' . $vat_number . '</p>';
		}
	}

	/**
	 * Outputs a hidden form field.
	 *
	 * @param string $field
	 * @param string $key
	 * @param mixed $args
	 * @param string $value (default: null)
	 *
	 * @since 1.5
	 */
	function wc_form_hidden_field( $field, $key, $args, $value ){
		$defaults = array(
			'label'             => '',
			'id'                => $key,
			'class'             => array(),
			'input_class'       => array(),
			'custom_attributes' => array(),
			'default'           => '',
		); 
		$args = wp_parse_args( $args, $defaults );

		// Custom attribute handling
		$custom_attributes = array();
		if ( ! empty( $args['custom_attributes'] ) && is_array( $args['custom_attributes'] ) )
			foreach ( $args['custom_attributes'] as $attribute => $attribute_value )
			$custom_attributes[] = esc_attr( $attribute ) . '="' . esc_attr( $attribute_value ) . '"';

		$field = '<p class="form-row ' . esc_attr( implode( ' ', $args['class'] ) ) .'" id="' . esc_attr( $args['id'] ) . '_field">';

		$field .= '<input type="hidden" class="input-text ' . esc_attr( implode( ' ', $args['input_class'] ) ) .'" name="' . esc_attr( $key ) . '" id="' . esc_attr( $args['id'] ) . '" value="' . esc_attr( $value ) . '" ' . implode( ' ', $custom_attributes ) . ' /></p>';

		return $field;
	}

	/**
	 * Validate a VAT number
	 *
	 * @param string $vat_number
	 * @param string $country
	 *
	 * @return boolean
	 *
	 * @since 1.9
	 */
	public static function is_valid( $vat_number, $country ){
    global $woocommerce;

    // get the country code from the number if it's empty
    if ( empty($country) ) {
      $country = substr( $vat_number, 0, 2 );
    }

	  $params = array(
			'vat_number' => $vat_number,
			'country' => $country
		);

		$slug = 'vat_number_' . md5( implode( $params ) );

		if ( false === ( $valid_number = get_transient( $slug ) ) ) {
			$valid_number = (int) QuadernoTax::validate( $params );
			set_transient( $slug, $valid_number, 4 * WEEK_IN_SECONDS );
		}

		return $valid_number == 1 && $country != $woocommerce->countries->get_base_country();
	}

  /**
   * Add custom fields to admin area
   *
   * @since 1.12
   */
  public function add_customer_meta_fields( $user ) {
    ?>    
    <h2><?php _e( 'Additional Information', 'woocommerce-quaderno' ); ?></h2>
    <table class="form-table" id="woocommerce-quaderno-meta">
      <tbody>
        <tr>
          <th>
            <label for="vat_number"><?php echo esc_html__( 'VAT number', 'woocommerce-quaderno' ) ?></label>
          </th>
          <td>
            <input type="text" name="vat_number" id="vat_number" value="<?php echo esc_attr( get_the_author_meta( '_quaderno_vat_number', $user->ID ) ); ?>" class="regular-text" />
          </td>
        </tr>
        <?php do_action('woocommerce_quaderno_meta_fields', $user ) ?>
      </tbody>
    </table>
    <?php
  }

  /**
   * Save custom fields from admin area
   *
   * @since 1.12
   */
  public function save_customer_meta_fields( $user_id ) {
    if ( !current_user_can( 'edit_user', $user_id ) ) { 
      return false; 
    }
    update_user_meta( $user_id, '_quaderno_vat_number', $_POST['vat_number'] );
  }
}